/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/* 
 * To see the console logs logged by this file, in the console view, enter the following:
 * gc.console.setCookie("scb.js", <value>) 
 * Where value: 0 = remove the cookie, 1 = only errors, 2 = errors and warnings, 3 = errors,warnings,info, 4 = errors,warnings,info,logs, 5=debug+all others
 * For more help on how to work with gc.console, please see the help text that is sent to the
 * console when the app starts.
 */
// if (!gc.designer) { gc.console.setCookie("scb.js", 5); }

// TODO: Remove from global scope


const NUM_BUFS_TO_CONCAT = 1; // number of buffers to concatenate before sending data up websocket
var initComplete = false;
var serviceUsb;
var usbDeviceOpen = false;
var msTimeout = 1000;
const keyPrefix = "1cbe_00ab";          // SCB
//const keyPrefix = "1cbe_0003";          // TIVA
//const keyPrefix = "1cbe_af81";          // USB-PHI
const DFU_deviceKey = "1cbe_00ff";      // MSP432E DFU Mode

var activeKey = "";
const interfaceIndex = 0; // USB bulk interface index


// Default packet handler
function defaultPacketHandler(rxData) {

	// Log data to console (NOTE: disable when measuring throughput)
	console.log(rxData);

	// CONSIDER: Measuring USB throughput here
}
var bulkPacketHdlr = defaultPacketHandler;


// Error handler
function pktErrHdlr(rxError) {
	var msg = JSON.stringify(rxError);
	console.error("scb.js", "pktErrHdlr error: '" + msg + "'");
}


function openBulkDevice(deviceKey) {

	gc.console.log("scb.js", "Connecting to USB BULK device: " + deviceKey);

	// Open and claim USB interface
	let serviceUsb = gc.services['ti-service-usb'];
	serviceUsb.openUsbDevice(deviceKey).then(function (deviceInfo) {
		return serviceUsb.claimInterface(deviceKey, interfaceIndex, true, NUM_BUFS_TO_CONCAT);

	}).then(function () {

		SCB.bulkDeviceKey = deviceKey;
		SCB.bulkDeviceConnected = true;
		gc.console.debug("scb.js", "Device " + deviceKey + " interface claimed. Polling started. Num bufs concatenated = " + NUM_BUFS_TO_CONCAT + '\n');

		// Select the default packet handler
		bulkPacketHdlr = defaultPacketHandler.bind(this);
		return serviceUsb.addListener(deviceKey, interfaceIndex, bulkPacketHdlr, pktErrHdlr);

	}).fail(function (err) {
		console.error("scb.js", "Error opening BULK device: " + deviceKey + "', error=" + err.message);
	});
}

function changePacketHandler(newPacketHandler, scope) {

	const activeKey = SCB.bulkDeviceKey;
	let serviceUsb = gc.services['ti-service-usb'];

	return serviceUsb.removeListener(activeKey, interfaceIndex, bulkPacketHdlr, pktErrHdlr)
		.then(function () {
			bulkPacketHdlr = newPacketHandler.bind(scope || this);
			return serviceUsb.addListener(activeKey, interfaceIndex, bulkPacketHdlr, pktErrHdlr);
		}).fail(function (err) {
			console.error("scb.js", "Error in changePacketHandler: '" + err.message);
		});
}


function closeBulkDevice(isDetachEvent) {

	const activeKey = SCB.bulkDeviceKey;
	let serviceUsb = gc.services['ti-service-usb'];

	return serviceUsb.removeListener(activeKey, interfaceIndex, bulkPacketHdlr, pktErrHdlr).finally(function () {
		if (!isDetachEvent) {
			return serviceUsb.closeUsbDevice(activeKey);
		}

	}).fail(function (err) {
		console.error("scb.js", "Error closing device '" + activeKey + "', error=" + err.message);

	}).finally(function () {
		SCB.bulkDeviceConnected = false;
		SCB.bulkDeviceKey = "";
		gc.console.log("scb.js", "USB device with key " + activeKey + " removed.\n");
	});
}


// The following variables are contained within their own local function scope...
(function () {

	// TODO: Do we need a reference to 'template_obj', or can we simply listen to this event from the document object?

	var initComplete = false;
	var templateObj = document.querySelector('#template_obj');
	if (templateObj) { init(); }
	else { document.addEventListener('DOMContentLoaded', init.bind(this)); }

	// Wait for DOMContentLoaded event before trying to access the application template
	function init() {

		// Get reference to template object (in case we did not get reference earlier)
		templateObj = document.querySelector('#template_obj');

		// Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
		// before trying to access components in the application.
		templateObj.addEventListener('dom-change', function () {

			if (initComplete) { return; }
			initComplete = true;

			this.async(function () {

				// Enable logging
				const moduleName = "scb.js";
				if (gc.desktop.isDesktop()) { gc.console.setCookie(moduleName, 2); }
				gc.console.debug(moduleName, "Initializing...");

			}, 1);

		});

	};

})();